#####
##
# OrderedObjectManager / maik.jablonski@uni-bielefeld.de
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# This product uses modified codes from the OrderedFolder-Product
# written by Stephan Richter / copyrighted by iuveno AG
# Please read the LICENSE.txt!
##
####

from types import StringType
from Globals import ImageFile, DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.ObjectManager import ObjectManager
from OFS.CopySupport import CopyContainer
from DocumentTemplate.sequence import sort

oomPermission = 'Copy or Move'
ObjectManager.security = ClassSecurityInfo()

ObjectManager.manage_main = DTMLFile('dtml/manage_main', globals())

misc_ = {'up'     : ImageFile('www/up.gif', globals()),
         'down'   : ImageFile('www/down.gif', globals()),
         'top'    : ImageFile('www/top.gif', globals()),
         'bottom' : ImageFile('www/bottom.gif', globals())}
        

def get_object_position(self, id):
    """ Get the position of a particular object. """
    
    ids = self.objectIds()

    if id in ids:
        return ids.index(id)

ObjectManager.security.declareProtected(oomPermission, 'get_object_position')
ObjectManager.get_object_position=get_object_position    
    
    
def move_objects_by_positions(self, ids, delta):
    """ Move objects with ids by delta. """

    if type(ids) is StringType:
        ids = (ids,)

    min_position = 0
    counter = 0

    objects = list(self._objects)

    obj_dict = {}
    for obj in objects:
        obj_dict[obj['id']] = obj

    if delta > 0:
        ids = list(ids)
        ids.reverse()
        objects.reverse()

    for id in ids:
        object = obj_dict[id]

        old_position = objects.index(object)
        new_position = max( old_position - abs(delta), min_position )

        if new_position == min_position:
            min_position += 1

        if old_position <> new_position:
            objects.remove(object)
            objects.insert(new_position, object)
            counter += 1

    if counter > 0:
        if delta > 0:
            objects.reverse()
        self._objects = tuple(objects)

    return counter

ObjectManager.security.declareProtected(oomPermission, 'move_objects_by_positions')
ObjectManager.move_objects_by_positions=move_objects_by_positions


def move_objects_up(self, ids, delta=1):
    """ Move objects with ids up by delta. """
        
    return self.move_objects_by_positions(ids, -delta)

ObjectManager.security.declareProtected(oomPermission, 'move_objects_up')
ObjectManager.move_objects_up=move_objects_up


def move_objects_down(self, ids, delta=1):
    """ Move objects with ids down by delta. """
     
    return self.move_objects_by_positions(ids, delta)

ObjectManager.security.declareProtected(oomPermission, 'move_objects_down')
ObjectManager.move_objects_down=move_objects_down


def move_objects_to_top(self, ids):
    """ Move objects with ids to top. """
               
    return self.move_objects_by_positions(ids, -len(self._objects))

ObjectManager.security.declareProtected(oomPermission, 'move_objects_to_top')
ObjectManager.move_objects_to_top=move_objects_to_top


def move_objects_to_bottom(self, ids):
    """ Move objects with ids to bottom. """

    return self.move_objects_by_positions(ids, len(self._objects))

ObjectManager.security.declareProtected(oomPermission, 'move_objects_to_bottom')
ObjectManager.move_objects_to_bottom=move_objects_to_bottom


def manage_move_objects_up(self, ids=None, delta=1, REQUEST=None):
    """ Move objects with ids up by delta (default = 1). """

    if ids:
        self.move_objects_up(ids, delta)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))

ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_up')
ObjectManager.manage_move_objects_up=manage_move_objects_up


def manage_move_objects_down(self, ids=None, delta=1, REQUEST=None):
    """ Move objects with ids down by delta (default = 1). """

    if ids:
        self.move_objects_down(ids, delta)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))


ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_down')
ObjectManager.manage_move_objects_down=manage_move_objects_down


def manage_move_objects_to_top(self, ids=None, REQUEST=None):
    """ Move objects with ids to top. """
    
    if ids: 
        self.move_objects_to_top(ids)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))


ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_to_top')
ObjectManager.manage_move_objects_to_top=manage_move_objects_to_top


def manage_move_objects_to_bottom(self, ids=None, REQUEST=None):
    """ Move objects with ids to bottom. """

    if ids:
        self.move_objects_to_bottom(ids)

    if REQUEST: 
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))


ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_to_bottom')
ObjectManager.manage_move_objects_to_bottom=manage_move_objects_to_bottom


def manage_orderObjects(self, key, reverse, REQUEST=None):
    """ Orders a objectmanager by key / reverse. """

    ids = [ id for id, obj in sort( self.objectItems(),
                                  ( (key, 'cmp', 'asc'), ) ) ]
    if reverse:
        ids.reverse()
    
    self.move_objects_to_top(ids)

    if REQUEST:
        REQUEST.RESPONSE.redirect(REQUEST.get('manage_redirect','manage_main'))

        
ObjectManager.security.declareProtected(oomPermission, 'manage_orderObjects')
ObjectManager.manage_orderObjects = manage_orderObjects

# Here we patch the CopyContainer - 
# manage_renameObject should preserve the order

CopyContainer._old_manage_renameObject=CopyContainer.manage_renameObject

def manage_renameObject(self, id, new_id, REQUEST=None):
    """ rename a particular sub-object """
    
    try:
        position = self.get_object_position(id)
        result = self._old_manage_renameObject(id, new_id, REQUEST)
        newposition = self.get_object_position(new_id)
        self.move_objects_by_positions(new_id, position - newposition)
    except:
        result = self._old_manage_renameObject(id, new_id, REQUEST)
    
    return result

CopyContainer.manage_renameObject=manage_renameObject

InitializeClass(ObjectManager) 


# And now try to register OrderedObjectManager for CMF/Plone

try:
    from Products.CMFCore.DirectoryView import registerDirectory
    
    global cmfoomanager_globals
    cmfoomanager_globals=globals()

    def initialize(context):
        registerDirectory('skins', cmfoomanager_globals)
except:
    pass
